/* See dg_spline.h */
/* Last edited on 2004-08-17 23:23:04 by stolfi */

#include <dg_tree.h>
#include <dg_spline.h>
#include <affirm.h>
#include <math.h>

double dg_tent_eval(dg_Dim d, double *c, dg_Rank r, double *x)
  { affirm(FALSE, "not implemented");
    return 0.0;
  }


dg_Tent_vec_t dg_maximal_tent_basis(dg_Tree *t)
  { affirm(FALSE, "not implemented");
    return dg_tent_vec_new(0);
  }


dg_Tent_vec_t dg_tent_vec_new(nat nel)
  { /* This is not a macro only because gcc does not allow cast of struct: */
    vec_t v = vec_new(nel, sizeof(dg_Tent));
    dg_Tent_vec_t r;
    r.nel = v.nel; r.el = (dg_Tent *)v.el;
    return r;
  }


dg_Tent_pair_vec_t dg_tent_pair_vec_new(nat nel)
  { /* This is not a macro only because gcc does not allow cast of struct: */
    vec_t v = vec_new(nel, sizeof(dg_Tent_pair));
    dg_Tent_pair_vec_t r;
    r.nel = v.nel; r.el = (dg_Tent_pair *)v.el;
    return r;
  }


dg_Tent_vec_t dg_minimal_tent_basis(dg_Tree *t)
{ 
  nat ntv = 0;
  dg_Tent_vec_t tv = dg_tent_vec_new(100); 
  int_vec_t perp_axis = int_vec_new(t->d);
  
  auto void dg_find_tents ( dg_Rank r, nat s, dg_Node **C );
  
  void dg_find_tents ( dg_Rank r, nat s, dg_Node **C )
  {
    int i, mask;
    int num_cells = (unsigned char)pow(2,s);
    dg_Node **CX = (dg_Node **)malloc(num_cells*sizeof(dg_Node *));
    dg_Axis long_axis = r % t->d;
    bool_t none_is_leaf, parallel_split;

    for( i=0; i<num_cells; i++ ) C[i] = C[i];    

    none_is_leaf = TRUE;
    for( i=0; i<num_cells; i++ ) 
      if( !(C[i]->ch[0] && C[i]->ch[1]) ) 
        { none_is_leaf = FALSE; } 

    if( none_is_leaf ) // There are no leaves
    {
      parallel_split = TRUE; // Check parallelism

      for( i=0; i<s; i++ ) 
        if( long_axis == perp_axis.el[i] ) 
        { 
          parallel_split = FALSE;
          mask = 1; 
          mask <<= (s - 1) - i;
        }
  
      if( parallel_split ) // It's parallel
      { 
        for( i=0; i<num_cells; i++ ) CX[i] = C[i]->ch[0];
        dg_find_tents ( r+1, s, CX );

        for( i=0; i<num_cells; i++ ) CX[i] = C[i]->ch[1];
        dg_find_tents ( r+1, s, CX );
 
        perp_axis.el[s] = long_axis; 
        for( i=0; i<num_cells; i++ )
        { CX[2*i] = C[i]->ch[0];
          CX[2*i+1] = C[i]->ch[1];
        }
        dg_find_tents ( r+1, s+1, CX );

      }
      else
      {
        for( i=0; i<num_cells; i++ )
        { 
          if( (i & mask) != 0 ) CX[i] = C[i]->ch[0];
          else CX[i] = C[i]->ch[1];
        }
        dg_find_tents ( r+1, s, CX);
      }
    }
    else if( s == t->d )
      { dg_tent_vec_expand(&tv, ntv);
        tv.el[ntv] = (dg_Tent){C[0]->index, r};
        ntv++;
      } 
    free(CX);  
  }
  
  dg_find_tents ( 0, 0, &(t->root) );
  dg_tent_vec_trim(&tv, ntv);
  return tv;
}


dg_Tent_pair_vec_t dg_minimal_tent_pairs(dg_Tree *t)
{ 
  nat ntpv = 0;
  dg_Tent_pair_vec_t tpv = dg_tent_pair_vec_new(2000); 
  dg_Rank r_ini[2];
  nat s_ini[2];
  dg_Node ** C_ini[2];
  int *perp_ini[2], i;
    
  auto void dg_find_tent_pairs(dg_Rank *rank, nat *S, dg_Node ***C, int **Perp, int cont);
  
  void dg_find_tent_pairs(dg_Rank *rank, nat *S, dg_Node ***C, int **Perp, int cont) // Variables come in 2, one for each element
  {
    int i, j, mask, divide, other;
    //    int num_cells[2], *perp_axis, *Next_perp[2];
    int num_cells[2], perp_axis[2], *Next_perp[2];
    dg_Rank r[2];
    nat s[2];
    // dg_Node **CX, **Next_els[2];
    dg_Node *CX[4], **Next_els[2];
    dg_Axis long_axis;
    bool_t none_is_leaf[2], parallel_split;

    for(i = 0; i < 2; i++) num_cells[i] = (unsigned char)pow(2,S[i]);
       
    if(dg_common_cells_found(rank, C, num_cells))
    { 
      for(i = 0; i < 2; i++)
      {  none_is_leaf[i] = TRUE;
         for( j = 0; j < num_cells[i]; j++ ) 
           if( !(C[i][j]->ch[0] && C[i][j]->ch[1]) ){ none_is_leaf[i] = FALSE; }     
      }
 
      printf("** Cont: %d ", cont); 
      for(i = 0; i < 2; i++) 
      { printf("** Cells %d ", i);
        for( j = 0; j < num_cells[i]; j++ ) 
          printf(" %d: %ld ", j, C[i][j]->index);
        printf("\n");
      }
      printf("\n");

      if( !(none_is_leaf[0] || none_is_leaf[1]) )
      { 
        if (cont == 3233130)printf(" TESTE 1 cont : %d s0: %d s1: %d \n", cont, S[0] ,S[1]);

        if( (S[0] == t->d) && (S[1] == t->d) )
        { dg_tent_pair_vec_expand(&tpv, ntpv);

	//          if (cont == 3233130)
	//          { printf(" cont : %d I0: %d I1: %d \n", cont, C[0][0]->index, C[1][0]->index);
	//            tpv.el[ntpv].cx[0] = 20;
	//            tpv.el[ntpv].cx[1] = 20;
	//            printf(" V0: %d V1: %d \n", tpv.el[ntpv].cx[0], tpv.el[ntpv].cx[1]);
	//          }else{ 
          tpv.el[ntpv].cx[0] = C[0][0]->index;
          tpv.el[ntpv].cx[1] = C[1][0]->index;
		//	  }
	  tpv.el[ntpv].r[0] = rank[0];
	  tpv.el[ntpv].r[1] = rank[1];
          ntpv++;     
        } 
        return;
      }  

      divide = 0; 
      if(rank[1] < rank[0]) divide = 1;  // The element with greater cells shall be divided

      if(!none_is_leaf[divide]) divide = (divide + 1) % 2; // Unless the element with greater cells has a leaf
      other = (divide + 1) % 2;

      //CX = (dg_Node **)malloc(num_cells[divide]*sizeof(dg_Node *));
      r[0] = rank[0]; r[1] = rank[1];
      s[0] = S[0]; s[1] = S[1];
    
      long_axis = rank[divide] % t->d;
      Next_els[other] = C[other]; Next_els[divide] = CX;
      r[divide]++; // All new elements shall have increased ranks
      
      parallel_split = TRUE; // Check parallelism

      for( i=0; i<s[divide]; i++ ) 
      { perp_axis[i] = Perp[divide][i];
        if( long_axis == Perp[divide][i] )
        { 
          parallel_split = FALSE;
          mask = 1; 
          mask <<= (s[divide] - 1) - i;
        }
      }

      if( parallel_split ) // It's parallel
      {         
        //perp_axis = (int *)malloc(t->d*sizeof(int));

        Next_perp[other] = Perp[other]; Next_perp[divide] = perp_axis;
 
        for( i=0; i<num_cells[divide]; i++ ) CX[i] = C[divide][i]->ch[0];
        cont++;
        dg_find_tent_pairs(r, s, Next_els, Next_perp, cont*10);
    
        for( i=0; i<num_cells[divide]; i++ ) CX[i] = C[divide][i]->ch[1];
        cont++;
        dg_find_tent_pairs(r, s, Next_els, Next_perp, cont*10);
         
        perp_axis[s[divide]] = long_axis; 
        for( i=0; i<num_cells[divide]; i++ )
        { CX[2*i] = C[divide][i]->ch[0];
          CX[2*i+1] = C[divide][i]->ch[1];
        }
        s[divide]++;
        cont++;
        dg_find_tent_pairs(r, s, Next_els, Next_perp, cont*10);
        //free(perp_axis);
      }
      else
      {
        for( i=0; i<num_cells[divide]; i++ )
        { 
          if( (i & mask) != 0 ) CX[i] = C[divide][i]->ch[0];
          else CX[i] = C[divide][i]->ch[1];
        }
        cont++;
        dg_find_tent_pairs(r, s, Next_els, Perp, cont*10);
        
      } 

    } 

    //free(CX);
  }
  
  for(i = 0; i < 2; i++)
  { r_ini[i] = 0; s_ini[i] = 0; C_ini[i] = &(t->root); } 
  
  dg_find_tent_pairs ( r_ini, s_ini, C_ini, perp_ini , 0);
  printf(" SIZEOF: %d \n", sizeof(dg_Tent_pair));
  printf(" SIZEOF: %d \n", sizeof(dg_Tent));

  dg_tent_pair_vec_trim(&tpv, ntpv);
  return tpv;

}



bool_t dg_common_cells_found(dg_Rank *r, dg_Node ***C, int *num_cells)
{
  int i, j, shorter = 0, greater;
  dg_Index I;

  if(r[0] == r[1])
  {
    for(i = 0; i < num_cells[0]; i++)
      for(j = 0; j < num_cells[1]; j++) if(C[0][i]->index == C[1][j]->index) return TRUE;
    return FALSE;
  }

  if(r[1] < r[0]) shorter = 1; 
  greater = (shorter + 1) % 2;
   
  for(i = 0; i < num_cells[shorter]; i++)
      for(j = 0; j < num_cells[greater]; j++)
      { 
        I = C[greater][j]->index;
        do
	{ 
          I = (int)I / 2;
	  if(C[shorter][i]->index == I) return TRUE;
        }while(I > 1);
      }
  return FALSE;

}
