/* Last edited on 2004-08-17 23:23:21 by stolfi */
/* Test of libdgrid routines. */

#include <stdio.h>
#include <math.h>
#include <dg_tree.h>
#include <dg_spline.h>
#include <affirm.h>
#include <ps.h>

#define DIM 3

typedef enum{X, Y, Z, T} dimensions;

//typedef enum{false, true} bool_t;

#define NUM_AXIS 2
typedef enum{x_axis, y_axis, z_axis} axis;

#define next_axis(P) ((P+1)%NUM_AXIS)

typedef struct dg_dimensions {
  double min[DIM];
  double max[DIM];
} dg_dimensions;
  
typedef bool_t split_criterion(dg_dimensions*);

void dg_begin_figure(
  FILE *f,
  dg_dimensions* dim
);

void dg_end_figure(FILE *f);

void dg_plot_2d_tree(
  FILE *f, dg_Tree *t, 
  dg_dimensions* dim,
  axis long_axis,
  int deep_max
);

void dg_plot_2d_subtree(
  FILE *f, dg_Node *p, 
  dg_dimensions* dim,
  axis long_axis,
  int deep,
  int deep_max
);

void dg_plot_grid_point(
  FILE *psfile, dg_Index index,
  dg_dimensions *dim, int r, int G, int B
);

double length(double dx, double dy, double dz);

bool_t toobig(dg_dimensions* dim);

void dg_build_tree_test(dg_Tree *t);

void dg_shatter_node(
  dg_Node *p, int maxlevel,
  split_criterion split,
  dg_dimensions* dim,
  axis long_axis
  );

int main()
{ 
  int i;
  dg_Tree *T; 

  dg_Tent_pair_vec_t tpv; 

  FILE *f = open_write("gridplot.eps");

  dg_dimensions dim, dim0, dim1;

  axis long_axis = x_axis;
  int deep_max = 14;

  dim.min[X] = 0.0;
  dim.max[X] = 1.0;
  dim.min[Y] = 0.0;
  dim.max[Y] = sqrt(2.0)/2;
  dim.min[Z] = 0.0;
  dim.max[Z] = 0.0;


  fprintf(stderr,"\n*\n*\n*\n");
  
  T = dg_Tree_new(2);

  //dg_shatter_node(T->root, 11, toobig, &dim, long_axis);
  
  dg_build_tree_test(T);

  dg_begin_figure(f, &dim);
  
  fprintf(stderr, "\n\nIso Start" );

  dg_plot_2d_tree(f, T, &dim, long_axis,deep_max); 

  //tv = dg_minimal_tent_basis(T);
  tpv = dg_minimal_tent_pairs(T);
  printf(" ******* VETOR -> NEL: %d \n", tpv.nel);

  for(i = 0; i < tpv.nel; i++) 
    { printf(" ******* NOVA TENDA -> A: %ld B: %ld \n", tpv.el[i].cx[0], tpv.el[i].cx[1]);
      dg_plot_grid_point(f, tpv.el[i].cx[0], &dim0, 0, 1, 0);
      dg_plot_grid_point(f, tpv.el[i].cx[1], &dim1, 0, 0, 1);
      ps_draw_segment(f, dim0.max[X], dim0.max[Y], dim1.max[X], dim1.max[Y]);
    }
  
  dg_end_figure(f);

  dg_Tree_free(T);  
  
  fprintf(stderr, "\n\nOK so far...\n");

  return(0);

}


void dg_plot_grid_point(
  FILE *psfile, dg_Index index,
  dg_dimensions *dim, int R, int G, int B
)
{
  int aux=0, tam, i;   		
  axis long_axis = x_axis;

  dim->min[X] = 0.0;
  dim->max[X] = 1.0;
  dim->min[Y] = 0.0;
  dim->max[Y] = sqrt(2.0)/2;
  dim->min[Z] = 0.0;
  dim->max[Z] = 0.0;

  tam = sizeof(index)*8;

  i=tam;
  while(aux==0)        /* Searches for the first bit 1 */ 
  {
    aux=index;
    i--;
    aux>>=i;
  }

  index<<=tam-i;       /* Clears the first bit 1 */
  index>>=tam-i;

  i--;

  while(i >= 0)
  {
    aux=index;
    aux<<=tam-i-1;
    aux>>=tam-1;
    
    if(aux) dim->min[long_axis] = (dim->min[long_axis] + dim->max[long_axis])/2;
    else dim->max[long_axis] = (dim->min[long_axis] + dim->max[long_axis])/2;
    long_axis = (long_axis + 1) % 2;

    i--;
  }

  // ps_set_pen(psfile, 1.0, 0.0, 0.0, 0.2, 0.0, 0.0); // Plots 1st cell in red 
  ps_fill_dot(psfile, dim->max[X], dim->max[Y], 0.8, R, G, B); // Plots tent dot in black 

}


double length(double dx, double dy, double dz)
{
  return sqrt(dx*dx + dy*dy + dz*dz);
}


bool_t toobig(dg_dimensions* dim)
/*
  An arbitrary splitting criterion. */
{
  double xctr = (dim->min[X] + dim->max[X])/2.0;
  double yctr = (dim->min[Y] + dim->max[Y])/2.0;
  double r = length((dim->max[X] - dim->min[X])/2.0, (dim->max[Y] - dim->min[Y])/2.0, 0.0);
  double d1 = length((xctr - 0.2)/9.0, (yctr - 0.3)/9.0, 0.003);
  double d2 = length((xctr - 0.6)/5.0, (yctr - 0.5)/5.0, 0.003);
  double d3 = length((yctr - (0.9 + 0.4*sin(2.0*xctr)))/4.0, 0.0, 0.003);
  double maxr = 3.0/(1.0/d1 + 1.0/d2 + 1.0/d3);
  return (r > maxr);
}



void dg_begin_figure(
  FILE *f,
  dg_dimensions* dim
)
/*  Prepares an Encapsulated Postscript file for drawing a dyadic grid. */
{
  double dx = dim->max[X] - dim->min[X];
  double ex = 0.1*dx;
  double toth = (dx + 2*ex)*450.0;
  
  double dy = dim->max[Y] - dim->min[Y];
  double ey = 0.1*dy;
  double totv = (dy + 2*ey)*450.0;
  
  ps_begin_figure(f, 0.0, toth, 0.0, totv);
  ps_set_window(f, 
    dim->min[X]-ex, dim->max[X]+ex, dim->min[Y]-ey, dim->max[Y]+ey,
    0.0, toth, 0.0, totv,
    5,7
  );
}



void dg_end_figure(FILE *f)
/*
  Terminates an Encapsulated Postscript drawing of a dyadic grid. */
{
  ps_end_figure(f);
}


  
void dg_plot_2d_tree(
  FILE *f, dg_Tree *t, 
  dg_dimensions* dim,
  axis long_axis,
  int deep_max
)
{
  affirm(t->d == 2, "wrong tree dimension");
  ps_set_pen(f, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0);
  ps_draw_rectangle(f, dim->min[X], dim->max[X], dim->min[Y], dim->max[Y]);
  dg_plot_2d_subtree(f, t->root, dim, long_axis, 1, deep_max);
}


  
void dg_plot_2d_subtree(
  FILE *f, dg_Node *p, 
  dg_dimensions* dim,
  axis long_axis,
  int deep,
  int deep_max
)
{
  dg_dimensions d;

  d.min[X] = dim->min[X];
  d.max[X] = dim->max[X];
  d.min[Y] = dim->min[Y];
  d.max[Y] = dim->max[Y];

  if ((p != NULL) && ((p->ch[0] != NULL) || (p->ch[1] != NULL)) && deep < deep_max)
    { 
      double xmid = (dim->max[X] + dim->min[X])/2.0;
      double ymid = (dim->max[Y] + dim->min[Y])/2.0;

      if (long_axis == x_axis)
      { 
          ps_draw_segment(f, xmid, dim->min[Y], xmid, dim->max[Y]);
	  d.max[X] = xmid;
          dg_plot_2d_subtree(f, p->ch[0], &d, y_axis, deep + 1, deep_max);

	  d.max[X] = dim->max[X];
	  d.min[X] = xmid;
          dg_plot_2d_subtree(f, p->ch[1], &d, y_axis, deep + 1, deep_max);
      }
      else
      { 
          ps_draw_segment(f, dim->min[X], ymid, dim->max[X], ymid);
	  d.max[Y] = ymid;
          dg_plot_2d_subtree(f, p->ch[0], &d, x_axis, deep + 1, deep_max);

	  d.max[Y] = dim->max[Y];
	  d.min[Y] = ymid;

          dg_plot_2d_subtree(f, p->ch[1], &d, x_axis, deep + 1, deep_max);
      }
    }
}



void dg_shatter_node(
  dg_Node *p, int maxlevel,
  split_criterion split,
  dg_dimensions* dim,
  axis long_axis

 )
/*
  Given a node with no children, replaces it
  by a random subtree of depth at most maxlevel. */
{
  dg_dimensions d;

  d.min[X] = dim->min[X];
  d.max[X] = dim->max[X];
  d.min[Y] = dim->min[Y];
  d.max[Y] = dim->max[Y];

  if ((maxlevel > 1) && (split(dim)))
    {
      dg_Node_split(p);
     
      if (long_axis == x_axis)
	{
	  double xmid = (dim->max[X] + dim->min[X])/2.0;
	  d.max[X] = xmid;
	  dg_shatter_node(p->ch[0], maxlevel-1, split, &d, y_axis);
	  d.max[X] = dim->max[X];
	  d.min[X] = xmid;
	  dg_shatter_node(p->ch[1], maxlevel-1, split, &d, y_axis);
	}
      else
	{
	  double ymid = (dim->max[Y] + dim->min[Y])/2.0;
	  d.max[Y] = ymid;
	  dg_shatter_node(p->ch[0], maxlevel-1, split, &d, x_axis);
	  d.max[Y] = dim->max[Y];
	  d.min[Y] = ymid;
	  dg_shatter_node(p->ch[1], maxlevel-1, split, &d, x_axis);
	}
     }
  
}



void dg_build_tree_test(dg_Tree *t)
{ int tam, i, inc = 0;
  dg_Index new_index, aux;
  dg_Node* p;

  tam = sizeof(new_index)*8;
  printf(" * Type the index of a new node: \n");
  scanf("%ld",&new_index);

  while(new_index!=0)
  {
    aux=0;   		/* Searches for the first bit 1 */
    i=tam;
    while(aux==0)
    {
      aux=new_index;
      i--;
      aux>>=i;
    }

    new_index<<=tam-i;       /* Clears the first bit 1 */
    new_index>>=tam-i;
    i--;

    p=t->root;
    while(i >= 0)
    {
      aux=new_index;
      aux<<=tam-i-1;
      aux>>=tam-1;

      if(aux)p=p->ch[1];
      else p=p->ch[0];
     i--;
    }
    dg_Node_split(p);
   
    printf(" * Type the index of a existing node: \n");
    scanf("%ld",&new_index);
    printf(" * Type the side to append it (0 | 1): \n");
    scanf("%d",&inc);
    new_index = new_index*2 + inc;
    printf(" *** The NEW node's index -> %ld \n" , new_index);
  }
}
