/* See dg_locus.h */
/* Last edited on 2005-06-05 15:44:34 by stolfi */

#include <dg_locus.h>
#include <dg_grid.h>
#include <dg_tree.h>
#include <affirm.h>
#include <set32.h>
#include <math.h>

/* EXPORTED PROCS */

/* LOCUS IN A MULTIGRID */

dg_dim_t dg_locus_dimension(dg_dim_t d, dg_locus_t E)
  { return d - set32_count(E.norm); }

dg_axis_t dg_locus_normal_axis(dg_dim_t d, dg_locus_t E, dg_axis_index_t j)
  { return set32_elem_from_index(E.norm,j); }

dg_axis_t dg_locus_spanning_axis(dg_dim_t d, dg_locus_t E, dg_axis_index_t j)
  { dg_axis_set_t ES = dg_axis_set_complement(d, E.norm);
    return set32_elem_from_index(ES,j);
  }

dg_locus_t dg_locus_minimize_rank(dg_dim_t d, dg_locus_t E)
  { dg_rank_t r = dg_cell_rank(E.cell);
    dg_axis_set_t NrmE = E.norm;
    while (set32_belongs((r+d-1) % d, NrmE) && ((E.cell & 1LL) == 0))
      { E.cell >>= 1; r--; }
    return E;
  }

/* FACES AND SUB-FACES OF LOCI */

dg_locus_t dg_locus_facet(dg_dim_t d, dg_locus_t E, dg_axis_t ax, box_signed_dir_t dir)
  {
    dg_axis_set_t da = 1 << ax;
    affirm(ax < d, "bad axis");
    if ((E.norm & da) != 0)
      { /* Locus {E} is already normal to axis {ax}: */
        return E;
      }
    else if (dir == box_NEG)
      { /* Just add {ax} to the normal set: */
        return dg_locus(E.norm | da, E.cell); 
      } 
    else if (dir == box_POS)
      { /* Must increment the cell index along axis {ax}: */
        return dg_locus(E.norm | da, dg_cell_shift(d,E.cell,ax,1)); 
      }
    else
      { /* Asked {dir == box_ZER}, means {E} itself: */
        return E; 
      } 
  }

dg_locus_t dg_locus_face(dg_dim_t d, dg_locus_t E, box_face_index_t fi)
  { int ax = 0;
    dg_axis_set_t ES = dg_axis_set_complement(d, E.norm);
    while (ES != 0)
      { if (ES & 1)
          { box_signed_dir_t dir = ((fi + 1) % 3) - 1; 
            if (dir != box_ZER) { E = dg_locus_facet(d, E, ax, dir); }
            fi /= 3;
          }
        ax++; ES >>= 1;
      }
    return E;
  }

dg_locus_t dg_locus_extend(dg_dim_t d, dg_locus_t E, dg_axis_t ax, box_signed_dir_t dir)
  {
    dg_axis_set_t da = 1 << ax;
    affirm(ax < d, "bad axis");
    if ((E.norm & da) == 0)
      { /* Locus {E} is already parallel to axis {ax}: */
        return E;
      }
    else if (dir == box_POS)
      { /* Just remove {ax} from the normal set: */
        return dg_locus(E.norm & (~da), E.cell); 
      } 
    else if (dir == box_NEG)
      { /* Must decrement the cell index along axis {ax}: */
        return dg_locus(E.norm & (~da), dg_cell_shift(d, E.cell, ax, -1)); 
      }
    else
      { /* Asked {dir == box_ZER}, means {E} itself: */
        return E; 
      } 
  }

dg_locus_t dg_locus_face_from_signature(dg_dim_t d, dg_locus_t E, box_signed_dir_t dir[])
  { dg_axis_index_t j = 0; dg_axis_t ax = 0;
    dg_axis_set_t ES = dg_axis_set_complement(d, E.norm);
    while (ES != 0)
      { if (ES & 1) 
          { box_signed_dir_t dirj = dir[j];
            if (dirj != box_ZER) { E = dg_locus_facet(d, E, ax, dirj); }
            j++;
          }
        ax++; ES >>= 1;
      }
    return E;
  }
  
void dg_locus_box_root_relative(dg_dim_t d, dg_locus_t E, interval_t B[])
  {
    int i;
    dg_cell_box_root_relative(d, E.cell, B);
    for (i = 0; i < d; i++)
      { if (set32_belongs(i, E.norm)) 
          { HI(B[i]) = LO(B[i]); }
      }
  }

void dg_locus_star_size(dg_dim_t d, dg_locus_t E, dg_grid_size_t psz[])
  { int i;
    for (i = 0; i < d; i++)
      { psz[i] = (set32_belongs(i, E.norm) ? 2 : 1); }
  }

void dg_locus_star(dg_dim_t d, dg_locus_t E, dg_cell_index_t k[])
  {
    dg_grid_pos_t pos[dg_MAX_GRID_DIM];
    dg_cell_position(d, E.cell, pos);
    int i, ix;
    k[0] = E.cell; ix = 1;
    for (i = 0; i < d; i++)
      { if (set32_belongs(i, E.norm))
          { /* Copy all cells already found, shift the rest down: */
            int ix0;
            for (ix0 = 0; ix0 < ix; ix0++)
              { k[ix + ix0] = k[ix0];
                k[ix0] = dg_cell_shift(d, k[ix0], i, -1);
              }
            ix = 2*ix;
          }
      }
  }

/* PRINTOUT */

void dg_locus_print(FILE *wr, dg_dim_t d, dg_locus_t E)
  { 
    int i;
    dg_grid_pos_t Ep[dg_MAX_GRID_DIM];
    dg_cell_position(d, E.cell, Ep);
    dg_cell_print(wr, d, E.cell);
    fprintf(wr, ":");
    for (i = 0; i < d; i++) 
      { fprintf(wr, "%d", (set32_belongs(i, E.norm) ? 1 : 0)); }
  }

/* VECTOR ALLOCATION */

dg_locus_vec_t dg_locus_vec_new(unsigned int nel)
  { /* This is not a macro only because gcc does not allow cast of struct: */
    vec_t v = vec_new(nel, sizeof(dg_locus_t));
    dg_locus_vec_t r;
    r.nel = v.nel; r.el = (dg_locus_t *)v.el;
    return r;
  }
