/* B-pulses - B-Spline-like. */
/* Last edited on 2005-04-22 10:12:39 by stolfi */

#ifndef dg_pulse_B_B
#define dg_pulse_B_B

#include <dg_grid.h>
#include <bz_basic.h>
#include <dg_pulse.h>
#include <js.h>

/*
  B-PULSES
  
  The dyadic B-pulses are defined for any {c \geq -1}. 
  
  For a given {c}, there is exactly one mother pulse and degree
  {g = c+1} and. It is supported on {msz = c+2} intervals, 
  spanning {(0 _  msz)}.
  
  For {c = -1} the mother B-pulse is the unit constant function
  in the interval {(0_1)}.  For {c = 0} the B-pulse is the 
  same as the B-pulse and the H-pulse (a triangular function
  on {(0_2)}. For {c \geq 2} the B-pulses are the B-spline 
  basis.

      { w^g_c[m](z) = SUM { choose(c-k,m-k)/2^{c-k} BB^g_{k}(z) : k = 0..m } }

    For {z} in {[-1_0]}, the following relation holds

      { w^g_c[m](z) = w^g_c[c-m](-z) }
  */

unsigned int dg_pulse_B_num_mothers (dg_cont_t c, dg_degree_t g);
  /* Returns the number of mothers for B-pulses with continuity {c}
    and degree {g}: namely, 1 if {g = c+1}, 0 otherwise. */

dg_grid_size_t dg_pulse_B_mother_supp_count
  ( dg_cont_t c, 
    dg_degree_t g, 
    dg_pulse_mother_index_t m
  );
  /* Returns the support count of the mother B-pulse with given
    {c,g,m}: namely, {g+1 = c+2} if {g == c+1}, undefined
    otherwise. */
    
void dg_pulse_B_mother_to_bezier
  ( dg_cont_t c, 
    dg_degree_t g, 
    dg_pulse_mother_index_t m,
    dg_grid_pos_t x,
    double *bz
  );
  /* Stores into {bz[0..g]} the Bzier coefficients of the mother
    B-pulse with continuity {c}, degree {g}, and mother {m}, for the
    interval {(x _ x+1)}. Fails if such pulse is not defined or
    supported by this module. The coefficients will be non-zero only
    if {x == 0}, or if {x == -1} and {m <= c}. */

#endif
