#ifndef dg_spline_H
#define dg_spline_H

/* Polynomial splines in arbitrary dimension */
/* Last edited on 2005-04-22 09:52:28 by stolfi */

#include <stdint.h>

/*
  GENERAL SPLINES
  
  A /{d}-dimensional mesh/ is a collection of pairwise disjoint
  subsets of {R^d}, the /tiles/ of the mesh.
  
  Let {L} be a mesh, and {F} be a family of functions from {R^d} to
  {R}. An /{F}-spline over {L}/ is a function {s} from {D = \U L} to {R},
  such that {X*s = X*f_X} for every tile of {L}, where {f_X} is some
  function of {F}, and {X*f} denotes the restriction of {f} to {X}.
  Each partial function {X*f_X} is called a /piece/ of the spline.
  
  POLYNOMIAL SPLINES
  
  When the family {F} consists of the polynomials on the coordinates
  of {R^d}, with maximum degree {g} on each coordinate, we say that
  {s} is a /polynomial spline of degree g/. Splines can be assumed to
  be polynomial unless said otherwise.
  
  DYADIC F-SPLINES
  
  A spline is /dyadic/ if its mesh {L} is the set of leaf faces of
  some finite dyadic grid {G}.
  
  SPLINE BASES  
  
  We denote by {S(F,L)} the set of all {F}-splines over {L}. Note that
  the restriction of any function of {F} to {D = \U L} is also a
  spline of {S(F,L)}; and, if {F} is a finite-dimensional vector
  space, so is {S(F,L)}. In fact, if {J} is a basis for {F}, then the
  set
  
    {\set{ X*\phi : phi \in J \and X \in L }} 
    
  contains a basis for {S(F,L)}.
  
  The /{L}-support/ of a spline is the set of tiles of {L} whose
  pieces are not identically zero. Note that this set is generally
  larger than the set of /points/ where the spline is nonzero.
  
  FINITE ELEMENTS

  A /finite element/ is a spline with finite, and generally small, {L}-support. */
  
typedef int8_t dg_degree_t;
  /* Degree of a polynomial, spline, etc.. */
  
typedef int8_t dg_cont_t;
  /* Continuity order of a spline. Value {-1} means discontinuous.. */

#endif
