/* See dg_basis.h */
/* Last edited on 2005-02-21 11:27:32 by stolfi */

#include <dg_tent_basis.h> 

#include <dg_grid.h> 
#include <dg_tree.h>
#include <dg_locus.h>
#include <dg_tent.h> 
#include <affirm.h> 
#include <math.h>

/* INTERNAL PROTOTYPES */

/* IMPLEMENTATIONS */
dg_tent_id_vec_t dg_tent_basis_get
  ( dg_dim_t d, 
    dg_pulse_kind_t pkind[],
    dg_cont_t c[],
    dg_degree_t g[],
    dg_tree_t G
  )
  {
    /* Allocate the basis, initially empty: */
    dg_tent_id_vec_t B = dg_tent_id_vec_new(0);
    int nB = 0;
    
    /* Find maximum tent indices along each axis: */
    int mix[d];
    int i;
    for (i = 0; i < d; i++) 
      { mix[i] = dg_pulse_max_index(pkind[i], c[i], g[i]);
        demand(mix[i] >= 0, "bad pulse kind/c/g combination");
      }
    /* Compute the maximum tent index: */
    dg_tent_index_t mtix = dg_tent_index_max(d, mix);
    /* Enumerate basis, by template shape: */
    bool_t tixok[mtix+1]; /* Tent indices that have been gathered. */
    int tix;
    for (tix = 0; tix <= mtix; tix++) { tixok[tix] = FALSE; }
    dg_pulse_index_t pix[d], pixb[d];  /* Pulse indices for some tent index. */
    dg_grid_size_t psz[d], pszb[d];    /* Mother pulse sizes. */
    for (tix = 0; tix < mtix; tix++)
      { if (! tixok[tix])
          { /* Get pulse indices from tent index: */
            dg_tent_pulse_indices(d, mix, tix, pix);
            /* Get template {psz[0..d-1]} for these pulses: */
            dg_tent_mother_supp_size(d, pkind, c, g, pix, psz);
            /* Find cells that fit this template with leaf tree packs: */
            /* Should use {dg_tree_pack-enum} directly !!! */
            dg_cell_index_vec_t C = dg_tree_pack_get(d, G, psz, TRUE);
            /* Find all tent indices that also use this same template: */
            int tixb;
            for (tixb = 0; tixb <= mtix; tixb++)
              { if (! tixok[tixb])
                  { /* Get pulse indices from tent index: */
                    dg_tent_pulse_indices(d, mix, tixb, pixb);
                    /* Check if their template is {psz[0..d-1]}: */
                    dg_tent_mother_supp_size(d, pkind, c, g, pixb, pszb);
                    bool_t same = TRUE;
                    for (i = 0; i < d; i++)  { same &= (pszb[i] == psz[i]); }
                    if (same)
                      { /* Add tent index {tixb} with all cells in {C}: */
                        int ic;
                        for (ic = 0; ic < C.nel; ic++)
                          { dg_tent_id_vec_expand(&B, nB);
                            B.el[nB] = (dg_tent_id_t){ tixb, C.el[i] };
                            nB++;
                          }
                        /* Housecleaning: */
                        /* Mark thi stent index as gathered: */
                        tixok[tixb] = TRUE;
                      }
                  }
              }
          }
      }
    /* Return basis: */
    dg_tent_id_vec_trim(&B, nB);
    return B;
  }
